@@ -122,17 +122,6 @@ sub BUILD {
     $self->get_api_response();
 }
 
-=head2 list_xml
-
- return xml output
-=cut
-
-sub list_xml {
-    my $self = shift;
- 
-    return $self->xml_output;
-}
-
 =head2 list
 
  return cluster's attributes text output from hash_output attribute
