@@ -1,13 +1,11 @@
 package Ovirt::VM;
 
 use v5.10;
-use LWP::UserAgent;
-use XML::LibXML;
 use Carp;
 use Moo;
 
 with 'Ovirt';
-our $VERSION = '0.03';
+our $VERSION = '0.04';
 
 =head1 NAME
 
@@ -15,7 +13,7 @@ Ovirt::VM - Bindings for oVirt VM API
 
 =head1 VERSION
 
-Version 0.03
+Version 0.04
 
 =cut
 
@@ -33,8 +31,9 @@ Version 0.03
 
  my $vm = Ovirt::VM->new(%con);
 
- # return xml output
+ # return xml / json output
  print $vm->list_xml; 
+ print $vm->list_json;
  
  # list vm separated by new line
  # return attributes specified on 'vm_output_attrs'
@@ -120,7 +119,9 @@ Version 0.03
 
 has 'vm_url'                => ( is => 'ro', default => '/api/vms' );
 has 'vm_output_attrs'       => ( is => 'rw', default => 'id,name,state',
-                                 isa => sub {
+                                 isa => 
+                                 sub 
+                                 {
                                      # store all output attribute into array split by ','
                                      # $_[0] is the arguments spefied during initialization
                                      my @attrs = split ',' => $_[0];
@@ -143,7 +144,8 @@ has 'vm_output_attrs'       => ( is => 'rw', default => 'id,name,state',
                                                                 timezone        usb_enabled
                                                                 host_id         display_host_subject
                                                             |;
-                                     for my $attr (@attrs) {
+                                     for my $attr (@attrs) 
+                                     {
                                          $attr = lc ($attr);
                                          $attr = Ovirt->trim($attr);
                                          croak "Attribute $attr is not valid / supported"
@@ -155,7 +157,8 @@ has 'vm_output_delimiter'   => ( is => 'rw', default => '||' );
 has 'vm_cdrom_xml'          => ( is => 'ro', default => '<action><vm><os><boot dev="cdrom"/></os></vm></action>');
 has 'vm_hd_xml'             => ( is => 'ro', default => '<action><vm><os><boot dev="hd"/></os></vm></action>');
 has 'vm_boot_dev'           => ( is => 'rw', 
-                                 isa => sub { 
+                                 isa => sub 
+                                        { 
                                                 my $dev = $_[0];
                                                 $dev    = lc ($dev);
                                                 
@@ -174,39 +177,32 @@ has 'vm_boot_dev'           => ( is => 'rw',
  check if output attributes is valid
 =cut
 
-sub BUILD {
+sub BUILD 
+{
     my $self = shift;
     
     $self->pass_log_obj;
     
-    if ($self->id) {
+    if ($self->id) 
+    {
         $self->_set_root_url($self->vm_url . '/' . $self->id);
     }
-    else {
+    else 
+    {
         $self->_set_root_url($self->vm_url);
     }
     
     $self->get_api_response();
 }
 
-=head2 list_xml
-
- return xml output
-=cut
-
-sub list_xml {
-    my $self = shift;
-    
-    return $self->xml_output;
-}
-
 =head2 vm_action_output
 
  this method handle the output e.g start / stop vm
  required arguments ($xml), output passed by start/stop method
 =cut
 
-sub vm_action_output {
+sub vm_action_output 
+{
     my $self    = shift;
     
     # xml output from action (start,stop,reboot, etc)
@@ -223,24 +219,29 @@ sub vm_action_output {
 
 =cut
 
-sub remove {
+sub remove 
+{
     my $self = shift;
     
     my $vmid = shift || undef;
     my $move_url;
     
     # set the move final url
-    if ($self->id) {
+    if ($self->id) 
+    {
         $move_url = $self->url;
     }
-    else {
-        if ($vmid) {
+    else 
+    {
+        if ($vmid) 
+        {
             my $is_valid = $self->is_vmid_valid($vmid);
             croak "vm id not found" unless $is_valid;
             
             $move_url = $self->url . "/$vmid";
         }
-        else {
+        else 
+        {
             croak "vm id is required";
         }
     }
@@ -257,6 +258,121 @@ sub remove {
     $self->vm_action_output($output);
 }
 
+=head2 list_snapshot
+
+ list vm's snapshot
+ required argument ('vm id')
+
+=cut
+
+sub list_snapshot 
+{
+    my $self    = shift;
+    my $vmid    = shift || undef;
+    
+    my $snapshot_url;
+    
+    # don't proceed if vm not exist
+    my $is_valid = $self->is_vmid_valid($vmid);
+    croak "vm id not found" 
+        unless $is_valid;
+    
+    # set the url
+    $snapshot_url = $self->url . "/$vmid/snapshots";
+    
+    $self->get_api_output($snapshot_url);
+}
+
+=head2 create_snapshot
+
+ required arguments ('vm id', 'snapshot description', 'memorystate')
+ memorystate can be true/false
+ 
+ $vm->create_snapshot('ec191b8c-b3ec-41b9-9f9f-6d92bc32732b', 'test3', 'false');
+
+=cut
+
+sub create_snapshot
+{
+    my $self    = shift;
+    
+    my ($vmid, $description, $memorystate)  = @_;
+    
+    # don't proceed if vm not exist
+    my $is_valid = $self->is_vmid_valid($vmid);
+    croak "vm id not found" 
+        unless $is_valid;
+    
+    # set the url to be post
+    my $snapshot_url = $self->url . "/$vmid/snapshots";
+    
+    croak "memory state not supported, use true/false." 
+        unless $memorystate =~ /\b(true|false)\b/;
+        
+    croak "snapshot description required"
+        unless $description;
+        
+    # set xml string to be post
+    my $xml = <<EOF;
+<snapshot>
+    <description>$description</description>
+    <persist_memorystate>$memorystate</persist_memorystate>
+    <vm id="$vmid"></vm>
+</snapshot>
+EOF
+    
+    $self->log->debug($xml);
+    
+    # set user agent
+    my $ua      = LWP::UserAgent->new();
+    my $action  = $ua->post($snapshot_url, Content_Type => 'application/xml', Content => $xml);
+    
+    my $parser = XML::LibXML->new();
+    my $output = $parser->parse_string($action->decoded_content);
+    
+    $self->log->debug($output);
+    
+    return $output;
+}
+
+=head2 delete_snapshot
+
+ required arguments ('vm id', 'snapshot id')
+ $vm->delete_snapshot('ec191b8c-b3ec-41b9-9f9f-6d92bc32732b', 'b0e9198c-a404-4659-8a82-421ae85f54e1');
+
+=cut
+
+sub delete_snapshot
+{
+    my $self    = shift;
+    
+    my ($vmid, $snapshot_id) = @_;
+    
+    my $snapshot_url;
+    
+    # don't proceed if vm not exist
+    my $is_valid = $self->is_vmid_valid($vmid);
+    croak "vm id not found" 
+        unless $is_valid;
+    
+    # set the url to be post
+    $snapshot_url = $self->url . "/$vmid/snapshots/$snapshot_id";
+    
+    croak "snapshot id required"
+        unless $snapshot_id;
+    
+    $self->log->debug("delete snapshot action url = $snapshot_url");
+    
+    # set user agent
+    my $ua      = LWP::UserAgent->new();
+    my $action  = $ua->delete($snapshot_url);
+    
+    my $parser = XML::LibXML->new();
+    my $output = $parser->parse_string($action->decoded_content);
+    
+    $self->vm_action_output($output);
+}
+
 =head2 list_disk
 
  list vm's disk
@@ -264,7 +380,8 @@ sub remove {
 
 =cut
 
-sub list_disk {
+sub list_disk 
+{
     my $self    = shift;
     my $vmid    = shift || undef;
     
@@ -278,15 +395,7 @@ sub list_disk {
     # set the url
     $disk_url = $self->url . "/$vmid/disks";
     
-    my $ua      = LWP::UserAgent->new();
-    my $tx      = $ua->get($disk_url);
-    
-    my $parser = XML::LibXML->new();
-    my $output = $parser->parse_string($tx->decoded_content);
-    
-    $self->log->debug($output);
-    
-    return $output; 
+    $self->get_api_output($disk_url);
 }
 
 =head2 add_disk
@@ -316,7 +425,8 @@ sub list_disk {
 
 =cut
 
-sub add_disk {
+sub add_disk 
+{
     my $self = shift;
     
     my ($vmid, $driver, $format, $size, $disk_name, $storage_domain_id)  = @_;
@@ -385,7 +495,8 @@ EOF
  
 =cut
 
-sub remove_disk {
+sub remove_disk 
+{
     my $self = shift;
     
     my ($vmid, $disk_id) = @_;
@@ -422,7 +533,8 @@ sub remove_disk {
 
 =cut
 
-sub list_nic {
+sub list_nic 
+{
     my $self    = shift;
     my $vmid    = shift || undef;
     
@@ -436,15 +548,7 @@ sub list_nic {
     # set the url
     $nic_url = $self->url . "/$vmid/nics";
     
-    my $ua      = LWP::UserAgent->new();
-    my $tx      = $ua->get($nic_url);
-    
-    my $parser = XML::LibXML->new();
-    my $output = $parser->parse_string($tx->decoded_content);
-    
-    $self->log->debug($output);
-    
-    return $output; 
+    $self->get_api_output($nic_url);
 }
 
 =head2 add_nic
@@ -465,7 +569,8 @@ sub list_nic {
 
 =cut
 
-sub add_nic {
+sub add_nic 
+{
     my $self = shift;
     
     my ($vmid, $driver, $nic_name, $network_name)  = @_;
@@ -525,7 +630,8 @@ EOF
  
 =cut
 
-sub remove_nic {
+sub remove_nic 
+{
     my $self = shift;
     
     my ($vmid, $nic_id) = @_;
@@ -569,7 +675,8 @@ sub remove_nic {
  
 =cut
 
-sub create {
+sub create 
+{
     my $self = shift;
     
     my ($vm_name, $cluster_name, $template_name, $memory) = @_;
@@ -588,7 +695,8 @@ sub create {
     # create xml to be post
     my $xml;
     
-    if ($memory) {
+    if ($memory) 
+    {
         $memory     = $self->trim($memory);
         croak "Memory did not look like number" unless $memory =~ /^\d+$/;
         
@@ -611,7 +719,8 @@ sub create {
 EOF
 
     }
-    else {
+    else 
+    {
         $self->log->debug("memory not specified");
         $xml = <<EOF;
 <vm>
@@ -653,24 +762,29 @@ EOF
  if $self->id is set during initialization, argument is not required
 =cut
 
-sub start {
+sub start 
+{
     my $self = shift;
     
     my $vmid = shift || undef;
     my $start_url;
     
     # set the start final url
-    if ($self->id) {
+    if ($self->id) 
+    {
         $start_url = $self->url . "/start";
     }
-    else {
-        if ($vmid) {
+    else 
+    {
+        if ($vmid) 
+        {
             my $is_valid = $self->is_vmid_valid($vmid);
             croak "vm id not found" unless $is_valid;
             
             $start_url = $self->url . "/$vmid/start";
         }
-        else {
+        else 
+        {
             croak "vm id is required";
         }
     }
@@ -681,10 +795,12 @@ sub start {
     my $ua      = LWP::UserAgent->new();
     my $action;
     
-    if ($self->vm_boot_dev eq 'hd') {
+    if ($self->vm_boot_dev eq 'hd') 
+    {
         $action = $ua->post($start_url, Content_Type => 'application/xml', Content => $self->vm_hd_xml);
     }
-    else {
+    else 
+    {
         $action = $ua->post($start_url, Content_Type => 'application/xml', Content => $self->vm_cdrom_xml);   
     }
     
@@ -701,23 +817,28 @@ sub start {
  if $self->id is set during initialization, argument is not required
 =cut
 
-sub stop {
+sub stop 
+{
     my $self = shift;
     
     my $vmid = shift || undef;
     my $stop_url;
     
     # set the stop final url
-    if ($self->id) {
+    if ($self->id) 
+    {
         $stop_url = $self->url . "/stop";
     }
-    else {
-        if ($vmid) {
+    else 
+    {
+        if ($vmid) 
+        {
             my $is_valid = $self->is_vmid_valid($vmid);
             croak "vm id not found" unless $is_valid;
             $stop_url = $self->url . "/$vmid/stop";
         }
-        else {
+        else 
+        {
             croak "vm id is required";
         }
     }
@@ -728,10 +849,12 @@ sub stop {
     my $ua      = LWP::UserAgent->new();
     my $action;
     
-    if ($self->vm_boot_dev eq 'hd') {
+    if ($self->vm_boot_dev eq 'hd') 
+    {
         $action = $ua->post($stop_url, Content_Type => 'application/xml', Content => $self->vm_hd_xml);
     }
-    else {
+    else 
+    {
         $action = $ua->post($stop_url, Content_Type => 'application/xml', Content => $self->vm_cdrom_xml);   
     }
     
@@ -750,7 +873,8 @@ sub stop {
  
 =cut
 
-sub stop_all {
+sub stop_all 
+{
     my $self = shift;
     
     # croak if id is defined because it will not return all vm
@@ -759,16 +883,19 @@ sub stop_all {
         
     my $running_vms = $self->get_running();
     
-    if ($running_vms) {
+    if ($running_vms) 
+    {
         my @vms = split '\n' => $running_vms;
         
-        for my $vm (@vms) {
+        for my $vm (@vms) 
+        {
             $self->log->debug($vm);
             my @vmid = split ',' => $vm;
             $self->stop($vmid[0]);
         }
     }
-    else {
+    else 
+    {
         print "no vm in up state\n";
         exit;
     }
@@ -783,7 +910,8 @@ sub stop_all {
  
 =cut
 
-sub get_running {
+sub get_running 
+{
     my $self    = shift;
     
     # store the output and return it at the end
@@ -796,13 +924,15 @@ sub get_running {
     my $last_element = pop (@attrs);
     $self->log->debug("last element = $last_element");
     
-    for my $element_id ( 0 .. $#{ $self->hash_output->{vm} } ) {
+    for my $element_id ( 0 .. $#{ $self->hash_output->{vm} } ) 
+    {
         # check the state first, if up, proceed
         $self->log->debug("requesting vm state for element $element_id");
         
-        if (my $state = $self->get_vm_by_element_id($element_id, 'state') || $self->not_available) {
-            
-            if ($state ne 'up') {
+        if (my $state = $self->get_vm_by_element_id($element_id, 'state') || $self->not_available) 
+        {   
+            if ($state ne 'up') 
+            {
                 $self->log->debug("not interested, vm state is = " . $state);
                 
                 # not interested if vm is not up, go to next loop
@@ -811,9 +941,10 @@ sub get_running {
         }
         
         # in case there's no any element left, the last element become the only attribute requested
-        if (@attrs) {
-            for my $attr (@attrs) {
-                
+        if (@attrs) 
+        {
+            for my $attr (@attrs) 
+            {
                 $self->log->debug("requesting attribute $attr for element $element_id");
 
                 my $attr_output = $self->get_vm_by_element_id($element_id, $attr) || $self->not_available;
@@ -825,7 +956,8 @@ sub get_running {
         #handle last element
         $self->log->debug("requesting attribute $last_element for element $element_id");
         
-        if (my $last_output = $self->get_vm_by_element_id($element_id, $last_element) || $self->not_available) {
+        if (my $last_output = $self->get_vm_by_element_id($element_id, $last_element) || $self->not_available) 
+        {
             $output .= $last_output . "\n";
             $self->log->debug("output for attribute $last_element element $element_id = " . $last_output);
         }
@@ -843,24 +975,29 @@ sub get_running {
  if $self->id is set during initialization, argument is not required
 =cut
 
-sub reboot {
+sub reboot 
+{
     my $self = shift;
     
     my $vmid = shift || undef;
     my $reboot_url;
     
     # set the reboot final url
-    if ($self->id) {
+    if ($self->id) 
+    {
         $reboot_url = $self->url . "/reboot";
     }
-    else {
-        if ($vmid) {
+    else 
+    {
+        if ($vmid) 
+        {
             my $is_valid = $self->is_vmid_valid($vmid);
             croak "vm id not found" unless $is_valid;
             
             $reboot_url = $self->url . "/$vmid/reboot";
         }
-        else {
+        else 
+        {
             croak "vm id is required";
         }
     }
@@ -871,10 +1008,12 @@ sub reboot {
     my $ua      = LWP::UserAgent->new();
     my $action;
     
-    if ($self->vm_boot_dev eq 'hd') {
+    if ($self->vm_boot_dev eq 'hd') 
+    {
         $action = $ua->post($reboot_url, Content_Type => 'application/xml', Content => $self->vm_hd_xml);
     }
-    else {
+    else 
+    {
         $action = $ua->post($reboot_url, Content_Type => 'application/xml', Content => $self->vm_cdrom_xml);   
     }
     
@@ -891,24 +1030,28 @@ sub reboot {
  if $self->id is set during initialization, argument is not required
 =cut
 
-sub suspend {
+sub suspend 
+{
     my $self = shift;
     
     my $vmid = shift || undef;
     my $suspend_url;
     
     # set the suspend final url
-    if ($self->id) {
+    if ($self->id) 
+    {
         $suspend_url = $self->url . "/suspend";
     }
     else {
-        if ($vmid) {
+        if ($vmid) 
+        {
             my $is_valid = $self->is_vmid_valid($vmid);
             croak "vm id not found" unless $is_valid;
             
             $suspend_url = $self->url . "/$vmid/suspend";
         }
-        else {
+        else 
+        {
             croak "vm id is required";
         }
     }
@@ -939,24 +1082,29 @@ sub suspend {
  if $self->id is set during initialization, argument is not required
 =cut
 
-sub migrate {
+sub migrate 
+{
     my $self = shift;
     
     my $vmid = shift || undef;
     my $migrate_url;
     
     # set the migrate final url
-    if ($self->id) {
+    if ($self->id) 
+    {
         $migrate_url = $self->url . "/migrate";
     }
-    else {
-        if ($vmid) {
+    else 
+    {
+        if ($vmid) 
+        {
             my $is_valid = $self->is_vmid_valid($vmid);
             croak "vm id not found" unless $is_valid;
             
             $migrate_url = $self->url . "/$vmid/migrate";
         }
-        else {
+        else 
+        {
             croak "vm id is required";
         }
     }
@@ -967,10 +1115,12 @@ sub migrate {
     my $ua      = LWP::UserAgent->new();
     my $action;
     
-    if ($self->vm_boot_dev eq 'hd') {
+    if ($self->vm_boot_dev eq 'hd') 
+    {
         $action = $ua->post($migrate_url, Content_Type => 'application/xml', Content => $self->vm_hd_xml);
     }
-    else {
+    else 
+    {
         $action = $ua->post($migrate_url, Content_Type => 'application/xml', Content => $self->vm_cdrom_xml);   
     }
     
@@ -985,7 +1135,8 @@ sub migrate {
     return false if vmid not valid
 =cut
 
-sub is_vmid_valid {
+sub is_vmid_valid 
+{
     my $self = shift;
     my $vmid = shift;
     
@@ -995,8 +1146,10 @@ sub is_vmid_valid {
     $self->log->debug("vm id = $vmid");
     
     # if vm id match, return 1
-    for my $element_id (0 .. $#{ $self->hash_output->{vm} }) {
-        if ($self->hash_output->{vm}[$element_id]->{id} eq $vmid) {
+    for my $element_id (0 .. $#{ $self->hash_output->{vm} }) 
+    {
+        if ($self->hash_output->{vm}[$element_id]->{id} eq $vmid) 
+        {
             $self->log->debug("$vmid is valid");
             return 1;
         }
@@ -1014,7 +1167,8 @@ sub is_vmid_valid {
 
 =cut
 
-sub get_statistics {
+sub get_statistics 
+{
     my $self    = shift;
     my $vmid    = $self->id || shift || die "vm id required\n";
     
@@ -1022,26 +1176,7 @@ sub get_statistics {
     my $ua      = LWP::UserAgent->new();
     
     $self->log->debug($url);
-    my $tx      = $ua->get($url);
-    
-    my $output;
-    
-    if ($tx->is_success) {
-        
-        local $XML::LibXML::skipXMLDeclaration = 1;
-        my $parser  = XML::LibXML->new();
-        $output     = $parser->parse_string($tx->decoded_content);
-    }
-    else {
-        my $err = $tx->status_line;
-        $self->log->debug("LWP Error : " . $err);
-        $self->log->debug("LWP Decoded Content :" . $tx->decoded_content);
-        
-        croak "LWP Status line : " . $err;
-        croak "LWP Decoded Content :" . $tx->decoded_content;
-    }
-    
-    return $output;
+    $self->get_api_output($url);
 }
 
 =head2 list
@@ -1054,7 +1189,8 @@ sub get_statistics {
  
 =cut
 
-sub list {
+sub list 
+{
     my $self = shift;
     
     my $vmid = shift || undef;
@@ -1072,8 +1208,10 @@ sub list {
     # if the id is defined during initialization
     # the rest api output will only contain attributes for this id
     # so it's not necessary to loop on vm element
-    if ($self->id) {
-        for my $attr (@attrs) {
+    if ($self->id) 
+    {
+        for my $attr (@attrs) 
+        {
             $self->log->debug("requesting attribute $attr");
     
             my $attr_output = $self->get_vm_by_self_id($attr) || $self->not_available;
@@ -1084,14 +1222,16 @@ sub list {
         #handle last element or the only element
         $self->log->debug("requesting attribute $last_element");
         
-        if (my $last_output = $self->get_vm_by_self_id($last_element) || $self->not_available) {
+        if (my $last_output = $self->get_vm_by_self_id($last_element) || $self->not_available) 
+        {
             $output .= $last_output;
             $self->log->debug("output for attribute $last_element  = " . $last_output);
         }
         
         $output .= "\n";
     }
-    elsif ($vmid) {
+    elsif ($vmid) 
+    {
         #store vmid element
         my $vmid_element;
         
@@ -1100,7 +1240,8 @@ sub list {
         # store hash to avoid keys on reference
         #my %hash = $self->hash_output->{vm};
         
-        for my $element_id ( 0 .. $#{ $self->hash_output->{vm} } ) {
+        for my $element_id ( 0 .. $#{ $self->hash_output->{vm} } ) 
+        {
             next unless 
                 $self->hash_output->{vm}[$element_id]->{id} eq $vmid;
             
@@ -1109,7 +1250,8 @@ sub list {
         
         croak "vm id not found" unless $vmid_element >= 0;
         
-        for my $attr (@attrs) { 
+        for my $attr (@attrs) 
+        { 
            $self->log->debug("requesting attribute $attr for element $vmid_element");
     
             my $attr_output = $self->get_vm_by_element_id($vmid_element, $attr) || $self->not_available;
@@ -1120,21 +1262,23 @@ sub list {
         #handle last element or the only element
         $self->log->debug("requesting attribute $last_element for element $vmid_element");
         
-        if (my $last_output = $self->get_vm_by_element_id($vmid_element, $last_element) || $self->not_available) {
+        if (my $last_output = $self->get_vm_by_element_id($vmid_element, $last_element) || $self->not_available) 
+        {
             $output .= $last_output;
             $self->log->debug("output for attribute $last_element element $vmid_element = " . $last_output);
         }
         
         $output .= "\n";
     }
-    else {
-        
-        for my $element_id ( 0 .. $#{ $self->hash_output->{vm} } ) {
-            
+    else 
+    { 
+        for my $element_id ( 0 .. $#{ $self->hash_output->{vm} } ) 
+        {
             # in case there's no any element left, the last element become the only attribute requested
-            if (@attrs) {
-                for my $attr (@attrs) {
-                    
+            if (@attrs) 
+            {
+                for my $attr (@attrs) 
+                {
                     $self->log->debug("requesting attribute $attr for element $element_id");
     
                     my $attr_output = $self->get_vm_by_element_id($element_id, $attr) || $self->not_available;
@@ -1146,7 +1290,8 @@ sub list {
             #handle last element or the only element
             $self->log->debug("requesting attribute $last_element for element $element_id");
             
-            if (my $last_output = $self->get_vm_by_element_id($element_id, $last_element) || $self->not_available) {
+            if (my $last_output = $self->get_vm_by_element_id($element_id, $last_element) || $self->not_available) 
+            {
                 $output .= $last_output;
                 $self->log->debug("output for attribute $last_element element $element_id = " . $last_output);
             }
@@ -1164,7 +1309,8 @@ sub list {
  An array element id and attribute name is required
 =cut
 
-sub get_vm_by_element_id {
+sub get_vm_by_element_id 
+{
     my $self = shift;
     
     my ($element_id, $attr) = @_;
@@ -1175,80 +1321,105 @@ sub get_vm_by_element_id {
     $attr = $self->trim($attr);    
     $self->log->debug("element id = $element_id, attribute = $attr");
     
-    if      ($attr eq 'id') {
+    if      ($attr eq 'id') 
+    {
             return $self->hash_output->{vm}[$element_id]->{id};
     }
-    elsif   ($attr eq 'name') {
+    elsif   ($attr eq 'name') 
+    {
             return $self->hash_output->{vm}[$element_id]->{name};
     }
-    elsif   ($attr eq 'memory') {
+    elsif   ($attr eq 'memory') 
+    {
             return $self->hash_output->{vm}[$element_id]->{memory};
     }
-    elsif   ($attr eq 'state') {
+    elsif   ($attr eq 'state') 
+    {
             return $self->hash_output->{vm}[$element_id]->{status}->{state};
     }
-    elsif   ($attr eq 'description') {
+    elsif   ($attr eq 'description') 
+    {
             return $self->hash_output->{vm}[$element_id]->{description};
     }
-    elsif   ($attr eq 'cpu_cores') {
+    elsif   ($attr eq 'cpu_cores') 
+    {
             return $self->hash_output->{vm}[$element_id]->{cpu}->{topology}->{cores};
     }
-    elsif   ($attr eq 'cpu_sockets') {
+    elsif   ($attr eq 'cpu_sockets') 
+    {
             return $self->hash_output->{vm}[$element_id]->{cpu}->{topology}->{sockets};
     }
-    elsif   ($attr eq 'cpu_arch') {
+    elsif   ($attr eq 'cpu_arch') 
+    {
             return $self->hash_output->{vm}[$element_id]->{cpu}->{architecture};
     }
-    elsif   ($attr eq 'cpu_shares') {
+    elsif   ($attr eq 'cpu_shares') 
+    {
             return $self->hash_output->{vm}[$element_id]->{cpu_shares};
     }
-    elsif   ($attr eq 'os_type') {
+    elsif   ($attr eq 'os_type') 
+    {
             return $self->hash_output->{vm}[$element_id]->{os}->{type};
     }
-    elsif   ($attr eq 'boot_dev') {
+    elsif   ($attr eq 'boot_dev') 
+    {
             return $self->hash_output->{vm}[$element_id]->{os}->{boot}->{dev};
     }
-    elsif   ($attr eq 'ha_enabled') {
+    elsif   ($attr eq 'ha_enabled') 
+    {
             return $self->hash_output->{vm}[$element_id]->{high_availability}->{enabled};
     }
-    elsif   ($attr eq 'ha_priority') {
+    elsif   ($attr eq 'ha_priority') 
+    {
             return $self->hash_output->{vm}[$element_id]->{high_availability}->{priority};
     }
-    elsif   ($attr eq 'display_type') {
+    elsif   ($attr eq 'display_type') 
+    {
             return $self->hash_output->{vm}[$element_id]->{display}->{type};
     }
-    elsif   ($attr eq 'display_address') {
+    elsif   ($attr eq 'display_address') 
+    {
             return $self->hash_output->{vm}[$element_id]->{display}->{address};
     }
-    elsif   ($attr eq 'display_port') {
+    elsif   ($attr eq 'display_port') 
+    {
             # spice will return secure_port
-            # vnc will return port
             return $self->hash_output->{vm}[$element_id]->{display}->{secure_port}
                 if $self->hash_output->{vm}[$element_id]->{display}->{secure_port};
+            
+            # vnc will return port    
             return $self->hash_output->{vm}[$element_id]->{display}->{port}
     }
-    elsif   ($attr eq 'display_host_subject') {
+    elsif   ($attr eq 'display_host_subject') 
+    {
             return $self->hash_output->{vm}[$element_id]->{display}->{certificate}->{subject};
     }
-    elsif   ($attr eq 'cluster_id') {
+    elsif   ($attr eq 'cluster_id') 
+    {
             return $self->hash_output->{vm}[$element_id]->{cluster}->{id};
     }
-    elsif   ($attr eq 'template_id') {
+    elsif   ($attr eq 'template_id') 
+    {
             return $self->hash_output->{vm}[$element_id]->{template}->{id};
     }
-    elsif   ($attr eq 'stop_time') {
+    elsif   ($attr eq 'stop_time') 
+    {
             return $self->hash_output->{vm}[$element_id]->{stop_time};
     }
-    elsif   ($attr eq 'creation_time') {
+    elsif   ($attr eq 'creation_time') 
+    {
             return $self->hash_output->{vm}[$element_id]->{creation_time};
     }
-    elsif   ($attr eq 'timezone') {
+    elsif   ($attr eq 'timezone') 
+    {
             return $self->hash_output->{vm}[$element_id]->{timezone};
     }
-    elsif   ($attr eq 'usb_enabled') {
+    elsif   ($attr eq 'usb_enabled') 
+    {
             return $self->hash_output->{vm}[$element_id]->{usb}->{enabled};
     }
-    elsif   ($attr eq 'host_id') {
+    elsif   ($attr eq 'host_id') 
+    {
             return $self->hash_output->{vm}[$element_id]->{host}->{id};
     }
 }
@@ -1260,7 +1431,8 @@ sub get_vm_by_element_id {
  attribute name is required
 =cut
 
-sub get_vm_by_self_id {
+sub get_vm_by_self_id 
+{
     my $self = shift;
     
     my $attr = shift;
@@ -1271,80 +1443,105 @@ sub get_vm_by_self_id {
     $attr = $self->trim($attr);    
     $self->log->debug("attribute = $attr");
     
-    if      ($attr eq 'id') {
+    if      ($attr eq 'id') 
+    {
             return $self->hash_output->{id};
     }
-    elsif   ($attr eq 'name') {
+    elsif   ($attr eq 'name') 
+    {
             return $self->hash_output->{name};
     }
-    elsif   ($attr eq 'memory') {
+    elsif   ($attr eq 'memory') 
+    {
             return $self->hash_output->{memory};
     }
-    elsif   ($attr eq 'state') {
+    elsif   ($attr eq 'state') 
+    {
             return $self->hash_output->{status}->{state};
     }
-    elsif   ($attr eq 'description') {
+    elsif   ($attr eq 'description') 
+    {
             return $self->hash_output->{description};
     }
-    elsif   ($attr eq 'cpu_cores') {
+    elsif   ($attr eq 'cpu_cores') 
+    {
             return $self->hash_output->{cpu}->{topology}->{cores};
     }
-    elsif   ($attr eq 'cpu_sockets') {
+    elsif   ($attr eq 'cpu_sockets') 
+    {
             return $self->hash_output->{cpu}->{topology}->{sockets};
     }
-    elsif   ($attr eq 'cpu_arch') {
+    elsif   ($attr eq 'cpu_arch') 
+    {
             return $self->hash_output->{cpu}->{architecture};
     }
-    elsif   ($attr eq 'cpu_shares') {
+    elsif   ($attr eq 'cpu_shares') 
+    {
             return $self->hash_output->{cpu_shares};
     }
-    elsif   ($attr eq 'os_type') {
+    elsif   ($attr eq 'os_type') 
+    {
             return $self->hash_output->{os}->{type};
     }
-    elsif   ($attr eq 'boot_dev') {
+    elsif   ($attr eq 'boot_dev') 
+    {
             return $self->hash_output->{os}->{boot}->{dev};
     }
-    elsif   ($attr eq 'ha_enabled') {
+    elsif   ($attr eq 'ha_enabled') 
+    {
             return $self->hash_output->{high_availability}->{enabled};
     }
-    elsif   ($attr eq 'ha_priority') {
+    elsif   ($attr eq 'ha_priority') 
+    {
             return $self->hash_output->{high_availability}->{priority};
     }
-    elsif   ($attr eq 'display_type') {
+    elsif   ($attr eq 'display_type') 
+    {
             return $self->hash_output->{display}->{type};
     }
-    elsif   ($attr eq 'display_address') {
+    elsif   ($attr eq 'display_address') 
+    {
             return $self->hash_output->{display}->{address};
     }
-    elsif   ($attr eq 'display_port') {
+    elsif   ($attr eq 'display_port') 
+    {
             # spice will return secure_port
-            # vnc will return port
             return $self->hash_output->{display}->{secure_port}
                 if $self->hash_output->{display}->{secure_port};
+                
+            # vnc will return port    
             return $self->hash_output->{display}->{port};
     }
-    elsif   ($attr eq 'display_host_subject') {
+    elsif   ($attr eq 'display_host_subject') 
+    {
             return $self->hash_output->{display}->{certificate}->{subject};
     }
-    elsif   ($attr eq 'cluster_id') {
+    elsif   ($attr eq 'cluster_id') 
+    {
             return $self->hash_output->{cluster}->{id};
     }
-    elsif   ($attr eq 'template_id') {
+    elsif   ($attr eq 'template_id') 
+    {
             return $self->hash_output->{template}->{id};
     }
-    elsif   ($attr eq 'stop_time') {
+    elsif   ($attr eq 'stop_time') 
+    {
             return $self->hash_output->{stop_time};
     }
-    elsif   ($attr eq 'creation_time') {
+    elsif   ($attr eq 'creation_time') 
+    {
             return $self->hash_output->{creation_time};
     }
-    elsif   ($attr eq 'timezone') {
+    elsif   ($attr eq 'timezone') 
+    {
             return $self->hash_output->{timezone};
     }
-    elsif   ($attr eq 'usb_enabled') {
+    elsif   ($attr eq 'usb_enabled') 
+    {
             return $self->hash_output->{usb}->{enabled};
     }
-    elsif   ($attr eq 'host_id') {
+    elsif   ($attr eq 'host_id') 
+    {
             return $self->hash_output->{host}->{id};
     }
 }
